@@ -158,18 +158,18 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             let exit_call = match shared_data_type {
                 Some(_) => match state_data_types.get(state_name) {
                     Some(_) => quote! {
-                        self.observer.#exit_fn_name(&mut ctx, &self.id, State::#next_state_name, &self.data, &self.state.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                        self.observer.#exit_fn_name(ctx, &self.id, State::#next_state_name, &self.data, &self.state.data).await.map_err(|e| TransitionError::ObserverError(e))?;
                     },
                     None => quote! {
-                        self.observer.#exit_fn_name(&mut ctx, &self.id, State::#next_state_name, &self.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                        self.observer.#exit_fn_name(ctx, &self.id, State::#next_state_name, &self.data).await.map_err(|e| TransitionError::ObserverError(e))?;
                     }
                 },
                 None => match state_data_types.get(state_name) {
                     Some(_) => quote! {
-                        self.observer.#exit_fn_name(&mut ctx, &self.id, State::#next_state_name, &self.state.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                        self.observer.#exit_fn_name(ctx, &self.id, State::#next_state_name, &self.state.data).await.map_err(|e| TransitionError::ObserverError(e))?;
                     },
                     None => quote! {
-                        self.observer.#exit_fn_name(&mut ctx, &self.id, State::#next_state_name).await.map_err(|e| TransitionError::ObserverError(e))?;
+                        self.observer.#exit_fn_name(ctx, &self.id, State::#next_state_name).await.map_err(|e| TransitionError::ObserverError(e))?;
                     }
                 }
             };
@@ -184,20 +184,20 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                 Some(a) => match shared_data_type {
                     Some(_) => quote! {
                         impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                            pub async fn #event(mut self, mut ctx: S, data: #a) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&mut ctx, &self.id, State::#state_name, State::#next_state_name, Some(&self.data), Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
+                            pub async fn #event(mut self, ctx: &mut S, data: #a) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
+                                self.observer.on_transition(ctx, &self.id, State::#state_name, State::#next_state_name, Some(&self.data), Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
-                                self.observer.#enter_fn_name(&mut ctx, &self.id, #enter_from_type, &self.data, &data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.#enter_fn_name(ctx, &self.id, #enter_from_type, &self.data, &data).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, S, T>::new(self.observer, self.id, #next_state_name::new(data), self.data))
                             }
                         }
                     },
                     None => quote! {
                         impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                            pub async fn #event(mut self, mut ctx: S, data: #a) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&mut ctx, &self.id, State::#state_name, State::#next_state_name, Option::<()>::None, Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
+                            pub async fn #event(mut self, ctx: &mut S, data: #a) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
+                                self.observer.on_transition(ctx, &self.id, State::#state_name, State::#next_state_name, Option::<()>::None, Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
-                                self.observer.#enter_fn_name(&mut ctx, &self.id, #enter_from_type, &data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.#enter_fn_name(ctx, &self.id, #enter_from_type, &data).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, S, T>::new(self.observer, self.id, #next_state_name::new(data)))
                             }
                         }
@@ -206,20 +206,20 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                 None => match shared_data_type {
                     Some(_) => quote! {
                         impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                            pub async fn #event(mut self, mut ctx: S) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&mut ctx, &self.id, State::#state_name, State::#next_state_name, Some(&self.data), Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
+                            pub async fn #event(mut self, ctx: &mut S) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
+                                self.observer.on_transition(ctx, &self.id, State::#state_name, State::#next_state_name, Some(&self.data), Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
-                                self.observer.#enter_fn_name(&mut ctx, &self.id, #enter_from_type, &self.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.#enter_fn_name(ctx, &self.id, #enter_from_type, &self.data).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, S, T>::new(self.observer, self.id, #next_state_name::new(), self.data))
                             }
                         }
                     },
                     None => quote! {
                         impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                            pub async fn #event(mut self, mut ctx: S) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&mut ctx, &self.id, State::#state_name, State::#next_state_name, Option::<()>::None, Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
+                            pub async fn #event(mut self, ctx: &mut S) -> Result<#parent_name<#next_state_name, S, T>, TransitionError<T::Error>> {
+                                self.observer.on_transition(ctx, &self.id, State::#state_name, State::#next_state_name, Option::<()>::None, Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
-                                self.observer.#enter_fn_name(&mut ctx, &self.id, #enter_from_type).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.#enter_fn_name(ctx, &self.id, #enter_from_type).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, S, T>::new(self.observer, self.id, #next_state_name::new()))
                             }
                         }
@@ -274,9 +274,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             #constructor
     
                             impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                                pub async fn init(mut ctx: S, mut observer: T, id: Option<String>, data: #sdt, state_data: #dt) -> Result<Self, InitError<T::Error>> {
-                                    let id = observer.on_init(&mut ctx, id, State::#state_name, Some(&data), Some(&state_data)).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
-                                    observer.#enter_fn_name(&mut ctx, &id, None, &data, &state_data).await.map_err(|e| InitError::ObserverError(e))?;
+                                pub async fn init(ctx: &mut S, mut observer: T, id: Option<String>, data: #sdt, state_data: #dt) -> Result<Self, InitError<T::Error>> {
+                                    let id = observer.on_init(ctx, id, State::#state_name, Some(&data), Some(&state_data)).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
+                                    observer.#enter_fn_name(ctx, &id, None, &data, &state_data).await.map_err(|e| InitError::ObserverError(e))?;
                                     Ok(Self::new(observer, id, #state_name::new(state_data), data))
                                 }
                             }
@@ -285,9 +285,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             #constructor
     
                             impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                                pub async fn init(mut ctx: S, mut observer: T, id: Option<String>, data: #sdt) -> Result<Self, InitError<T::Error>> {
-                                    let id = observer.on_init(&mut ctx, id, State::#state_name, Some(&data), Option::<()>::None).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
-                                    observer.#enter_fn_name(&mut ctx, &id, None, &data).await.map_err(|e| InitError::ObserverError(e))?;
+                                pub async fn init(ctx: &mut S, mut observer: T, id: Option<String>, data: #sdt) -> Result<Self, InitError<T::Error>> {
+                                    let id = observer.on_init(ctx, id, State::#state_name, Some(&data), Option::<()>::None).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
+                                    observer.#enter_fn_name(ctx, &id, None, &data).await.map_err(|e| InitError::ObserverError(e))?;
                                     Ok(Self::new(observer, id, #state_name::new(), data))
                                 }
                             }
@@ -318,9 +318,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             #constructor
     
                             impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                                pub async fn init(mut ctx: S, mut observer: T, id: Option<String>, state_data: #dt) -> Result<Self, InitError<T::Error>> {
-                                    let id = observer.on_init(&mut ctx, id, State::#state_name, Option::<()>::None, Some(&state_data)).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
-                                    observer.#enter_fn_name(&mut ctx, &id, None, &state_data).await.map_err(|e| InitError::ObserverError(e))?;
+                                pub async fn init(ctx: &mut S, mut observer: T, id: Option<String>, state_data: #dt) -> Result<Self, InitError<T::Error>> {
+                                    let id = observer.on_init(ctx, id, State::#state_name, Option::<()>::None, Some(&state_data)).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
+                                    observer.#enter_fn_name(ctx, &id, None, &state_data).await.map_err(|e| InitError::ObserverError(e))?;
                                     Ok(Self::new(observer, id, #state_name::new(state_data)))
                                 }
                             }
@@ -329,9 +329,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             #constructor
     
                             impl<S: Send, T: Observer<S> + Send> #parent_name<#state_name, S, T> {
-                                pub async fn init(mut ctx: S, mut observer: T, id: Option<String>) -> Result<Self, InitError<T::Error>> {
-                                    let id = observer.on_init(&mut ctx, id, State::#state_name, Option::<()>::None, Option::<()>::None).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
-                                    observer.#enter_fn_name(&mut ctx, &id, None).await.map_err(|e| InitError::ObserverError(e))?;
+                                pub async fn init(ctx: &mut S, mut observer: T, id: Option<String>) -> Result<Self, InitError<T::Error>> {
+                                    let id = observer.on_init(ctx, id, State::#state_name, Option::<()>::None, Option::<()>::None).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
+                                    observer.#enter_fn_name(ctx, &id, None).await.map_err(|e| InitError::ObserverError(e))?;
                                     Ok(Self::new(observer, id, #state_name::new()))
                                 }
                             }
@@ -578,9 +578,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             }
         }
 
-        pub async fn retrieve<S: Send, T: Retriever<S> + Observer<S> + Send>(mut ctx: S, mut retriever: T, id: String) -> Result<#wrapped_type<S, T>, RetrieveError<T::RetrieverError>> {
+        pub async fn retrieve<S: Send, T: Retriever<S> + Observer<S> + Send>(ctx: &mut S, mut retriever: T, id: String) -> Result<#wrapped_type<S, T>, RetrieveError<T::RetrieverError>> {
             let id_str: &str = &id;
-            let (state_string, maybe_data, maybe_state_data) = retriever.on_retrieve(&mut ctx, id_str).await.map_err(|e| RetrieveError::RetrieverError(e))?;
+            let (state_string, maybe_data, maybe_state_data) = retriever.on_retrieve(ctx, id_str).await.map_err(|e| RetrieveError::RetrieverError(e))?;
             restore(retriever, id, state_string, maybe_data, maybe_state_data).await.map_err(|e| RetrieveError::RestoreError(e))
         }
     };
